"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var ReactDOM = require("react-dom");
var react_dom_1 = require("react-dom");
var SteppableInput_1 = require("../SteppableInput/SteppableInput");
var Control_1 = require("../Control/Control");
var classnames = require("classnames");
var withTheme_1 = require("../../utils/withTheme");
var ButtonIcon_1 = require("../ButtonIcon/ButtonIcon");
var pure_1 = require("../../utils/pure");
var Popover_1 = require("../Popover/Popover");
var with_defaults_1 = require("../../utils/with-defaults");
var Option_1 = require("fp-ts/lib/Option");
var DateInput_model_1 = require("./DateInput.model");
exports.DATE_INPUT = Symbol('DateInput');
var RawDateInput = /** @class */ (function (_super) {
    tslib_1.__extends(RawDateInput, _super);
    function RawDateInput() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            isOpened: false,
        };
        _this.secondInput = false;
        _this.getSectionClassName = function (activeSection) {
            var _a;
            var _b = _this.props, theme = _b.theme, isDisabled = _b.isDisabled;
            var selectedActiveSection = _this.state.activeSection;
            return classnames(theme.section, (_a = {},
                _a[theme.section_isActive] = !isDisabled && selectedActiveSection === activeSection,
                _a));
        };
        _this.onIncrement = function () {
            _this.secondInput = false;
            var _a = _this.props.value, day = _a.day, month = _a.month, year = _a.year;
            var activeSection = _this.state.activeSection;
            switch (activeSection) {
                case DateInput_model_1.ActiveSection.Day: {
                    //day starts from 1 here and cannot be zero
                    var newDay = day.map(function (value) { return (value + 1) % 32 || 1; }).orElse(function () { return Option_1.some(1); });
                    _this.onValueChange(newDay, month, year);
                    break;
                }
                case DateInput_model_1.ActiveSection.Month: {
                    //month starts from 1 here and cannot be zero
                    var newMonth = DateInput_model_1.incrementMonthOption(month);
                    _this.onValueChange(day, newMonth, year);
                    break;
                }
                case DateInput_model_1.ActiveSection.Year: {
                    var newYear = year
                        .chain(function (value) {
                        if (value !== 9999) {
                            return Option_1.some(value + 1);
                        }
                        else {
                            return Option_1.none;
                        }
                    })
                        .orElse(function () { return Option_1.some(new Date().getFullYear()); });
                    _this.onValueChange(day, month, newYear);
                    break;
                }
            }
        };
        _this.onDecrement = function () {
            _this.secondInput = false;
            var _a = _this.props.value, day = _a.day, month = _a.month, year = _a.year;
            var activeSection = _this.state.activeSection;
            switch (activeSection) {
                case DateInput_model_1.ActiveSection.Day: {
                    //day starts from 1 and cannot be zero
                    var newDay = day.map(function (value) { return (value - 1) % 32 || 31; }).orElse(function () { return Option_1.some(31); });
                    _this.onValueChange(newDay, month, year);
                    break;
                }
                case DateInput_model_1.ActiveSection.Month: {
                    //month starts from 1 and cannot be zero
                    var newMonth = DateInput_model_1.decrementMonthOption(month);
                    _this.onValueChange(day, newMonth, year);
                    break;
                }
                case DateInput_model_1.ActiveSection.Year: {
                    var newYear = year
                        .chain(function (value) {
                        if (value !== 0) {
                            return Option_1.some(value - 1);
                        }
                        else {
                            return Option_1.none;
                        }
                    })
                        .orElse(function () { return Option_1.some(new Date().getFullYear()); });
                    _this.onValueChange(day, month, newYear);
                    break;
                }
            }
        };
        _this.onCalendarValueChange = function (date) {
            var _a = _this.props, onValueChange = _a.onValueChange, value = _a.value;
            _this.setState({
                isOpened: false,
            });
            if (date) {
                var selectedDate = DateInput_model_1.toObjectDate(date);
                if (DateInput_model_1.isDatesDifferent(value, selectedDate) && onValueChange) {
                    onValueChange({
                        day: selectedDate.day,
                        month: selectedDate.month,
                        year: selectedDate.year,
                    });
                }
            }
        };
        _this.onCalendarMouseDown = function (e) {
            //stop blur
            e.preventDefault();
        };
        _this.onClear = function () {
            _this.secondInput = false;
            _this.onValueChange(Option_1.none, Option_1.none, Option_1.none);
            var onClear = _this.props.onClear;
            onClear && onClear();
        };
        _this.onCalendarButtonMouseDown = function (e) {
            if (isDefined(_this.state.activeSection)) {
                e.preventDefault();
            }
        };
        _this.onDayMouseDown = function (e) {
            if (!_this.props.isDisabled) {
                _this.setState({
                    activeSection: DateInput_model_1.ActiveSection.Day,
                });
            }
        };
        _this.onMonthMouseDown = function (e) {
            if (!_this.props.isDisabled) {
                _this.setState({
                    activeSection: DateInput_model_1.ActiveSection.Month,
                });
            }
        };
        _this.onYearMouseDown = function (e) {
            if (!_this.props.isDisabled) {
                _this.setState({
                    activeSection: DateInput_model_1.ActiveSection.Year,
                });
            }
        };
        _this.onKeyDown = function (e) {
            var _a = _this.props.value, day = _a.day, month = _a.month, year = _a.year;
            var activeSection = _this.state.activeSection;
            switch (e.keyCode) {
                case Control_1.KeyCode.Escape: {
                    if (_this.state.isOpened) {
                        _this.setState({
                            isOpened: false,
                        });
                    }
                    break;
                }
                case Control_1.KeyCode.Left: {
                    e.preventDefault(); //block h-scrolling
                    switch (activeSection) {
                        case DateInput_model_1.ActiveSection.Month: //fallthrough
                        case DateInput_model_1.ActiveSection.Year: {
                            _this.secondInput = false;
                            break;
                        }
                    }
                    _this.selectPreviousSection();
                    break;
                }
                case Control_1.KeyCode.Right: {
                    e.preventDefault(); //block h-scrolling
                    switch (activeSection) {
                        case DateInput_model_1.ActiveSection.Day: //fallthrough
                        case DateInput_model_1.ActiveSection.Month: {
                            _this.secondInput = false;
                            break;
                        }
                    }
                    _this.selectNextSection();
                    break;
                }
                case Control_1.KeyCode.Delete: //fallthrough
                case Control_1.KeyCode.Backspace: {
                    _this.secondInput = false;
                    switch (activeSection) {
                        case DateInput_model_1.ActiveSection.Day: {
                            _this.onValueChange(Option_1.none, month, year);
                            break;
                        }
                        case DateInput_model_1.ActiveSection.Month: {
                            _this.onValueChange(day, Option_1.none, year);
                            break;
                        }
                        case DateInput_model_1.ActiveSection.Year: {
                            _this.onValueChange(day, month, Option_1.none);
                            break;
                        }
                    }
                    break;
                }
                default: {
                    var number = Control_1.KEY_CODE_NUM_MAP[e.keyCode];
                    if (isDefined(number)) {
                        _this.handleDigitKeyDown(number);
                    }
                }
            }
        };
        _this.onSteppableInputClick = function (e) {
            if (_this.state.isOpened) {
                var calendarButtonDOMNode = ReactDOM.findDOMNode(_this.calendarButtonRef);
                var thisNode = ReactDOM.findDOMNode(_this);
                if (calendarButtonDOMNode === null ||
                    calendarButtonDOMNode instanceof Text ||
                    thisNode === null ||
                    thisNode instanceof Text) {
                    return;
                }
                if (SteppableInput_1.checkParentsUpTo(e.target, calendarButtonDOMNode, thisNode)) {
                    //clicked on calendar button
                    _this.setState({
                        isOpened: false,
                    });
                }
                else {
                    _this.setState({
                        isOpened: true,
                    });
                }
            }
            else {
                _this.setState({
                    isOpened: true,
                });
            }
        };
        _this.onBlur = function (e) {
            _this.secondInput = false;
            _this.setState({
                activeSection: undefined,
                isOpened: false,
            });
            _this.props.onBlur && _this.props.onBlur();
        };
        _this.onFocus = function (e) {
            _this.secondInput = false;
            if (!isDefined(_this.state.activeSection)) {
                _this.setState({
                    activeSection: _this.getDefaultActiveSection(_this.props.dateFormatType),
                });
            }
            _this.props.onFocus && _this.props.onFocus();
        };
        _this.onMouseEnter = function (e) {
            _this.props.onMouseEnter && _this.props.onMouseEnter();
        };
        _this.onMouseLeave = function (e) {
            _this.props.onMouseLeave && _this.props.onMouseLeave();
        };
        return _this;
    }
    RawDateInput.prototype.render = function () {
        var _this = this;
        var _a;
        var _b = this.props, isDisabled = _b.isDisabled, error = _b.error, clearIcon = _b.clearIcon, calendarIcon = _b.calendarIcon, Calendar = _b.Calendar, incrementIcon = _b.incrementIcon, decrementIcon = _b.decrementIcon, theme = _b.theme, ButtonIcon = _b.ButtonIcon, SteppableInput = _b.SteppableInput, dateFormatType = _b.dateFormatType, _c = _b.value, day = _c.day, month = _c.month, year = _c.year;
        var yearClassName = this.getSectionClassName(DateInput_model_1.ActiveSection.Year);
        // check if "X" clear button should be visible - at least one part of date should be set
        var onClear = day.isSome() || month.isSome() || year.isSome() ? this.onClear : undefined;
        var innerClassName = classnames(theme.inner, (_a = {},
            _a[theme.inner_isFilled] = day.isSome() && month.isSome() && year.isSome(),
            _a));
        return (React.createElement(SteppableInput, { isDisabled: isDisabled, theme: theme.SteppableInput, onClear: onClear, error: error, decrementIcon: decrementIcon, incrementIcon: incrementIcon, clearIcon: clearIcon, onIncrement: this.onIncrement, onDecrement: this.onDecrement, onBlur: this.onBlur, onFocus: this.onFocus, onKeyDown: this.onKeyDown, onClick: this.onSteppableInputClick, onMouseEnter: this.onMouseEnter, onMouseLeave: this.onMouseLeave, innerRef: this.props.innerRef },
            React.createElement("div", { className: innerClassName },
                dateFormatType === DateInput_model_1.DateFormatType.DMY && this.renderDay(),
                dateFormatType === DateInput_model_1.DateFormatType.MDY && this.renderMonth(),
                React.createElement("span", { className: theme.separator }, "/"),
                dateFormatType === DateInput_model_1.DateFormatType.DMY && this.renderMonth(),
                dateFormatType === DateInput_model_1.DateFormatType.MDY && this.renderDay(),
                React.createElement("span", { className: theme.separator }, "/"),
                React.createElement("span", { className: yearClassName, onMouseDown: this.onYearMouseDown }, DateInput_model_1.format(year, DateInput_model_1.ActiveSection.Year))),
            Calendar &&
                calendarIcon && (React.createElement(ButtonIcon, { isFlat: true, ref: function (el) { return (_this.calendarButtonRef = el); }, isDisabled: isDisabled, tabIndex: -1, icon: calendarIcon, onMouseDown: this.onCalendarButtonMouseDown, theme: theme.CalendarButtonIcon })),
            this.renderCalendar()));
    };
    RawDateInput.prototype.renderDay = function () {
        var day = this.props.value.day;
        var dayClassName = this.getSectionClassName(DateInput_model_1.ActiveSection.Day);
        return (React.createElement("span", { className: dayClassName, onMouseDown: this.onDayMouseDown }, DateInput_model_1.format(day, DateInput_model_1.ActiveSection.Day)));
    };
    RawDateInput.prototype.renderMonth = function () {
        var month = this.props.value.month;
        var monthClassName = this.getSectionClassName(DateInput_model_1.ActiveSection.Month);
        var monthValue = month.map(function (value) { return value + 1; });
        return (React.createElement("span", { className: monthClassName, onMouseDown: this.onMonthMouseDown }, DateInput_model_1.format(monthValue, DateInput_model_1.ActiveSection.Month)));
    };
    RawDateInput.prototype.renderCalendar = function () {
        var _a = this.props, target = _a.target, Calendar = _a.Calendar, Popover = _a.Popover, _b = _a.value, day = _b.day, month = _b.month, year = _b.year, min = _a.min, max = _a.max, theme = _a.theme;
        var isOpened = this.state.isOpened;
        if (!Calendar) {
            return null;
        }
        var date = DateInput_model_1.buildDateOption(day)(month)(year);
        var calendar = (React.createElement(Calendar, { value: date.toNullable(), min: min, max: max, onMouseDown: this.onCalendarMouseDown, onValueChange: this.onCalendarValueChange }));
        if (target) {
            return react_dom_1.createPortal(isOpened ? calendar : null, target);
        }
        else {
            return (React.createElement(Popover, { theme: theme.Popover, anchor: this, closeOnClickAway: true, onMouseDown: this.onCalendarMouseDown, isOpened: isOpened }, calendar));
        }
    };
    RawDateInput.prototype.onValueChange = function (day, month, year) {
        var _a = this.props, onValueChange = _a.onValueChange, value = _a.value;
        DateInput_model_1.isDatesDifferent(value, { day: day, month: month, year: year }) && onValueChange && onValueChange({ day: day, month: month, year: year });
    };
    RawDateInput.prototype.getDefaultActiveSection = function (dateFormatType) {
        switch (dateFormatType) {
            case DateInput_model_1.DateFormatType.DMY: {
                return DateInput_model_1.ActiveSection.Day;
            }
            case DateInput_model_1.DateFormatType.MDY: {
                return DateInput_model_1.ActiveSection.Month;
            }
        }
    };
    RawDateInput.prototype.selectNextSection = function () {
        var activeSection = this.state.activeSection;
        var dateFormatType = this.props.dateFormatType;
        switch (activeSection) {
            case DateInput_model_1.ActiveSection.Day: {
                switch (dateFormatType) {
                    case DateInput_model_1.DateFormatType.DMY: {
                        this.setState({
                            activeSection: DateInput_model_1.ActiveSection.Month,
                        });
                        break;
                    }
                    case DateInput_model_1.DateFormatType.MDY: {
                        this.setState({
                            activeSection: DateInput_model_1.ActiveSection.Year,
                        });
                        break;
                    }
                }
                break;
            }
            case DateInput_model_1.ActiveSection.Month: {
                switch (dateFormatType) {
                    case DateInput_model_1.DateFormatType.DMY: {
                        this.setState({
                            activeSection: DateInput_model_1.ActiveSection.Year,
                        });
                        break;
                    }
                    case DateInput_model_1.DateFormatType.MDY: {
                        this.setState({
                            activeSection: DateInput_model_1.ActiveSection.Day,
                        });
                        break;
                    }
                }
                break;
            }
        }
    };
    RawDateInput.prototype.selectPreviousSection = function () {
        var activeSection = this.state.activeSection;
        var dateFormatType = this.props.dateFormatType;
        switch (activeSection) {
            case DateInput_model_1.ActiveSection.Day: {
                switch (dateFormatType) {
                    case DateInput_model_1.DateFormatType.MDY: {
                        this.setState({
                            activeSection: DateInput_model_1.ActiveSection.Month,
                        });
                        break;
                    }
                }
                break;
            }
            case DateInput_model_1.ActiveSection.Month: {
                switch (dateFormatType) {
                    case DateInput_model_1.DateFormatType.DMY: {
                        this.setState({
                            activeSection: DateInput_model_1.ActiveSection.Day,
                        });
                        break;
                    }
                }
                break;
            }
            case DateInput_model_1.ActiveSection.Year: {
                switch (dateFormatType) {
                    case DateInput_model_1.DateFormatType.DMY: {
                        this.setState({
                            activeSection: DateInput_model_1.ActiveSection.Month,
                        });
                        break;
                    }
                    case DateInput_model_1.DateFormatType.MDY: {
                        this.setState({
                            activeSection: DateInput_model_1.ActiveSection.Day,
                        });
                        break;
                    }
                }
                break;
            }
        }
    };
    RawDateInput.prototype.handleDigitKeyDown = function (digit) {
        var _a = this.props.value, day = _a.day, month = _a.month, year = _a.year;
        switch (this.state.activeSection) {
            case DateInput_model_1.ActiveSection.Day: {
                if (this.secondInput) {
                    var newDay = day.map(function (value) {
                        var dayValue = Number("" + value + digit);
                        if (value < 3) {
                            return dayValue;
                        }
                        else if (value === 3) {
                            return Math.min(dayValue, 31);
                        }
                        else {
                            return digit;
                        }
                    });
                    this.onValueChange(newDay, month, year);
                    this.selectNextSection();
                    this.secondInput = false;
                }
                else {
                    this.onValueChange(Option_1.some(digit), month, year);
                    if (digit > 3) {
                        this.selectNextSection();
                        this.secondInput = false;
                    }
                    else {
                        this.secondInput = true;
                    }
                }
                break;
            }
            case DateInput_model_1.ActiveSection.Month: {
                if (this.secondInput) {
                    var newMonth = month.map(function (value) {
                        var correctedMonth = value + 1;
                        var monthValue = Number("" + correctedMonth + digit) - 1;
                        if (correctedMonth < 1) {
                            return monthValue;
                        }
                        else if (correctedMonth === 1) {
                            return Math.min(monthValue, 11);
                        }
                        else {
                            return digit;
                        }
                    });
                    this.onValueChange(day, newMonth, year);
                    this.selectNextSection();
                    this.secondInput = false;
                }
                else {
                    this.onValueChange(day, Option_1.some(digit - 1), year);
                    if (digit > 1) {
                        this.selectNextSection();
                        this.secondInput = false;
                    }
                    else {
                        this.secondInput = true;
                    }
                }
                break;
            }
            case DateInput_model_1.ActiveSection.Year: {
                if (this.secondInput) {
                    var newYear = year.map(function (value) {
                        if (value < 1000) {
                            return Number("" + value + digit);
                        }
                        else {
                            return Number(("" + value + digit).substr(1));
                        }
                    });
                    this.onValueChange(day, month, newYear);
                }
                else {
                    this.onValueChange(day, month, Option_1.some(digit));
                    this.secondInput = true;
                }
                break;
            }
        }
    };
    RawDateInput = tslib_1.__decorate([
        pure_1.PURE
    ], RawDateInput);
    return RawDateInput;
}(React.Component));
exports.DateInput = withTheme_1.withTheme(exports.DATE_INPUT)(with_defaults_1.withDefaults({
    SteppableInput: SteppableInput_1.SteppableInput,
    ButtonIcon: ButtonIcon_1.ButtonIcon,
    Popover: Popover_1.Popover,
    dateFormatType: DateInput_model_1.DateFormatType.DMY,
})(RawDateInput));
function isDefined(value) {
    return typeof value !== 'undefined';
}
//# sourceMappingURL=DateInput.js.map