"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Option_1 = require("fp-ts/lib/Option");
var Setoid_1 = require("fp-ts/lib/Setoid");
var Apply_1 = require("fp-ts/lib/Apply");
var setoidOptionNumber = Option_1.getSetoid(Setoid_1.setoidNumber);
var dateSetoid = Setoid_1.getRecordSetoid({
    day: setoidOptionNumber,
    month: setoidOptionNumber,
    year: setoidOptionNumber,
});
exports.isDatesDifferent = function (x, y) { return !dateSetoid.equals(x, y); };
exports.buildDate = function (day) { return function (month) { return function (year) { return new Date(year, month, day); }; }; };
exports.buildDateOption = Apply_1.liftA3(Option_1.option)(exports.buildDate);
var DateFormatType;
(function (DateFormatType) {
    DateFormatType[DateFormatType["MDY"] = 0] = "MDY";
    DateFormatType[DateFormatType["DMY"] = 1] = "DMY";
})(DateFormatType = exports.DateFormatType || (exports.DateFormatType = {}));
var ActiveSection;
(function (ActiveSection) {
    ActiveSection[ActiveSection["Day"] = 0] = "Day";
    ActiveSection[ActiveSection["Month"] = 1] = "Month";
    ActiveSection[ActiveSection["Year"] = 2] = "Year";
})(ActiveSection = exports.ActiveSection || (exports.ActiveSection = {}));
exports.toObjectDate = function (date) { return ({
    day: Option_1.some(date.getDate()),
    month: Option_1.some(date.getMonth()),
    year: Option_1.some(date.getFullYear()),
}); };
function format(date, section) {
    return date.foldL(function () {
        switch (section) {
            case ActiveSection.Day: {
                return 'dd';
            }
            case ActiveSection.Month: {
                return 'mm';
            }
            case ActiveSection.Year: {
                return 'yyyy';
            }
        }
    }, function (value) {
        switch (section) {
            //maybe we should use left-pad here? ;)
            case ActiveSection.Day: //fallthrough
            case ActiveSection.Month: {
                //day and month are 2 digits
                return "" + (value >= 0 && value < 10 ? 0 : '') + value;
            }
            case ActiveSection.Year: {
                if (value < 10) {
                    return "000" + value;
                }
                else if (value < 100) {
                    return "00" + value;
                }
                else if (value < 1000) {
                    return "0" + value;
                }
                else {
                    return "" + value;
                }
            }
        }
    });
}
exports.format = format;
function decrementMonth(month) {
    if (month >= 0 && month <= 11) {
        return (month - 1 + 12) % 12;
    }
    return 11;
}
exports.decrementMonth = decrementMonth;
function decrementMonthOption(month) {
    return month.map(decrementMonth).alt(Option_1.some(11));
}
exports.decrementMonthOption = decrementMonthOption;
function incrementMonth(month) {
    if (month >= 0 && month <= 11) {
        return (month + 1) % 12;
    }
    return 0;
}
exports.incrementMonth = incrementMonth;
function incrementMonthOption(month) {
    return month.map(incrementMonth).alt(Option_1.some(0));
}
exports.incrementMonthOption = incrementMonthOption;
//# sourceMappingURL=DateInput.model.js.map