"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var ReactDOM = require("react-dom");
var prefix_1 = require("@devexperts/utils/dist/dom/prefix");
var classnames = require("classnames");
var pure_1 = require("../../utils/pure");
var BoundsUpdateDetector_1 = require("../BoundsUpdateDetector/BoundsUpdateDetector");
var throttle_1 = require("@devexperts/utils/dist/function/throttle");
var withTheme_1 = require("../../utils/withTheme");
var EventListener_1 = require("../EventListener/EventListener");
var RootClose_1 = require("../RootClose/RootClose");
var with_defaults_1 = require("../../utils/with-defaults");
var react_dom_1 = require("react-dom");
var Popover_model_1 = require("./Popover.model");
var Popover_model_2 = require("./Popover.model");
exports.PopoverPlacement = Popover_model_2.PopoverPlacement;
exports.PopoverAlign = Popover_model_2.PopoverAlign;
exports.POPOVER = Symbol('Popover');
var RawPopover = /** @class */ (function (_super) {
    tslib_1.__extends(RawPopover, _super);
    function RawPopover(props) {
        var _this = _super.call(this, props) || this;
        _this.state = {};
        _this._needsUpdate = false;
        _this.updatePosition = function () {
            var anchorRect = _this.getAnchorProperties();
            if (!anchorRect) {
                return;
            }
            var _a = _this.props, placement = _a.placement, align = _a.align, hasArrow = _a.hasArrow;
            var arrowOffset;
            var finalPlacement;
            var finalAlign;
            var topResult = Popover_model_1.movePopoverVertically({
                placement: placement,
                align: align,
                anchorTop: anchorRect.top,
                anchorBottom: anchorRect.bottom,
                popoverHeight: _this._popoverSize.height,
                parentHeight: anchorRect.parentHeight,
                checkBounds: true,
            });
            var leftResult = Popover_model_1.movePopoverHorizontally({
                placement: placement,
                align: align,
                anchorLeft: anchorRect.left,
                anchorRight: anchorRect.right,
                popoverWidth: _this._popoverSize.width,
                parentWidth: anchorRect.parentWidth,
                checkBounds: true,
            });
            //additional
            if (placement === Popover_model_1.PopoverPlacement.Top || placement === Popover_model_1.PopoverPlacement.Bottom) {
                finalPlacement = topResult.placement;
                finalAlign = leftResult.align;
                if (hasArrow) {
                    //horizontal arrow offset
                    arrowOffset = Math.round((anchorRect.right - anchorRect.left) / 2);
                }
            }
            else if (placement === Popover_model_1.PopoverPlacement.Left || placement === Popover_model_1.PopoverPlacement.Right) {
                finalPlacement = leftResult.placement;
                finalAlign = topResult.align;
                if (hasArrow) {
                    //vertical arrow offset
                    arrowOffset = Math.round((anchorRect.bottom - anchorRect.top) / 2);
                }
            }
            _this.setState({
                top: Math.round(topResult.top) + window.pageYOffset,
                left: Math.round(leftResult.left) + window.pageXOffset,
                finalPlacement: finalPlacement,
                finalAlign: finalAlign,
                arrowOffset: arrowOffset,
            });
        };
        _this.onSizeUpdate = function (newSize) {
            _this._popoverSize = newSize;
            _this.updatePosition();
        };
        _this.onResize = function () {
            _this.handleResize();
        };
        _this.onScroll = function (event) {
            if (_this._anchor && _this._popover && event.target) {
                var popoverNode = react_dom_1.findDOMNode(_this._popover);
                var scrolled = event.target;
                if (popoverNode && scrolled.contains(_this._anchor)) {
                    _this.handleScroll();
                }
            }
        };
        _this.rootElement = document.createElement('div');
        var container = props.container || document.body;
        container.appendChild(_this.rootElement);
        _this.handleResize = throttle_1.default(_this.handleResize, 100);
        _this.handleScroll = throttle_1.default(_this.handleScroll, 100);
        return _this;
    }
    RawPopover.prototype.componentDidMount = function () {
        var _this = this;
        if (this.props.isOpened) {
            if (this.props.anchor) {
                var anchorDOMNode = react_dom_1.findDOMNode(this.props.anchor);
                if (!anchorDOMNode || anchorDOMNode instanceof Text) {
                    return;
                }
                this._anchor = anchorDOMNode;
            }
            this._popoverSize = this.getPopoverSize();
            this.updatePosition();
        }
        if (window.matchMedia) {
            this.mediaQueryList = window.matchMedia('print');
            this.mediaQueryList.addListener(function () { return _this.updatePosition(); });
        }
        window['onbeforeprint'] = this.updatePosition;
        window['onafterprint'] = this.updatePosition;
    };
    RawPopover.prototype.componentWillUnmount = function () {
        var container = this.props.container || document.body;
        container.removeChild(this.rootElement);
        this.mediaQueryList && this.mediaQueryList.removeListener(this.updatePosition);
        window['onbeforeprint'] = null;
        window['onafterprint'] = null;
    };
    RawPopover.prototype.componentWillReceiveProps = function (nextProps) {
        if (nextProps.isOpened && nextProps.anchor) {
            this._needsUpdate = true;
            var anchorDOMNode = react_dom_1.findDOMNode(nextProps.anchor);
            if (!anchorDOMNode || anchorDOMNode instanceof Text) {
                return;
            }
            this._anchor = anchorDOMNode;
        }
    };
    RawPopover.prototype.componentDidUpdate = function (prevProps) {
        //@PURE check is passed here - update anyway
        if (this._needsUpdate) {
            /** set in {@link componentWillReceiveProps} */
            this._needsUpdate = false;
            this._popoverSize = this.getPopoverSize();
            this.updatePosition();
        }
    };
    RawPopover.prototype.render = function () {
        var _this = this;
        var _a;
        var _b = this.props, closeOnClickAway = _b.closeOnClickAway, theme = _b.theme, hasArrow = _b.hasArrow, onMouseDown = _b.onMouseDown, isOpened = _b.isOpened, onRequestClose = _b.onRequestClose, anchor = _b.anchor;
        if (!isOpened || !anchor) {
            return null;
        }
        var _c = this.state, top = _c.top, left = _c.left, arrowOffset = _c.arrowOffset, finalPlacement = _c.finalPlacement, finalAlign = _c.finalAlign;
        var isMeasured = !!finalAlign && !!finalPlacement;
        var style;
        var popoverClassName = classnames(theme.container);
        if (isMeasured && finalPlacement) {
            style = prefix_1.default({
                transform: "translate(" + (left || 0) + "px, " + (top || 0) + "px)",
            });
            popoverClassName = classnames(popoverClassName, (_a = {},
                _a[theme.container_hasArrow] = hasArrow,
                _a), [Popover_model_1.getPlacementModifier(finalPlacement)].map(function (mod) { return theme["container_" + mod]; }));
        }
        style = tslib_1.__assign({}, style, this.props.style);
        var child = (React.createElement(BoundsUpdateDetector_1.BoundsUpdateDetector, { onUpdate: this.onSizeUpdate },
            React.createElement("div", { ref: function (el) { return (_this._popover = el); }, style: style, onMouseDown: onMouseDown, onClick: stopPropagation, className: popoverClassName },
                React.createElement("div", { className: theme.content },
                    isMeasured &&
                        finalPlacement &&
                        finalAlign &&
                        hasArrow && (React.createElement("div", { className: theme.arrow, style: getArrowStyle(finalPlacement, finalAlign, arrowOffset) })),
                    this.props.children))));
        if (closeOnClickAway) {
            child = React.createElement(RootClose_1.RootClose, { onRootClose: onRequestClose }, child);
        }
        var target = typeof window !== 'undefined' ? window : 'window';
        return (React.createElement(EventListener_1.EventListener, { onResize: this.onResize, onScrollCapture: this.onScroll, target: target }, ReactDOM.createPortal(child, this.rootElement)));
    };
    RawPopover.prototype.getPopoverSize = function () {
        var popover = react_dom_1.findDOMNode(this._popover);
        if (popover instanceof HTMLElement) {
            return {
                height: popover.offsetHeight,
                width: popover.offsetWidth,
            };
        }
        return {
            height: 0,
            width: 0,
        };
    };
    RawPopover.prototype.getAnchorProperties = function () {
        var container = this.props.container;
        var anchor = this._anchor;
        if (container && anchor) {
            var containerRect = container.getBoundingClientRect();
            var anchorRect = anchor.getBoundingClientRect();
            return {
                left: anchorRect.left - containerRect.left,
                right: anchorRect.right - containerRect.left,
                top: anchorRect.top - containerRect.top,
                bottom: anchorRect.bottom - containerRect.top,
                width: anchorRect.width,
                parentWidth: containerRect.width,
                parentHeight: containerRect.height,
            };
        }
        else if (anchor) {
            var anchorRect = anchor.getBoundingClientRect();
            return {
                left: anchorRect.left,
                right: anchorRect.right,
                top: anchorRect.top,
                bottom: anchorRect.bottom,
                width: anchorRect.width,
                parentWidth: window.innerWidth,
                parentHeight: window.innerHeight,
            };
        }
        else {
            return;
        }
    };
    RawPopover.prototype.handleResize = function () {
        this.updatePosition();
    };
    RawPopover.prototype.handleScroll = function () {
        var _a = this.props, onRequestClose = _a.onRequestClose, isOpened = _a.isOpened, disableCloseOnScroll = _a.disableCloseOnScroll;
        if (onRequestClose && isOpened && !disableCloseOnScroll) {
            onRequestClose();
        }
    };
    RawPopover = tslib_1.__decorate([
        pure_1.PURE,
        tslib_1.__metadata("design:paramtypes", [Object])
    ], RawPopover);
    return RawPopover;
}(React.Component));
exports.Popover = withTheme_1.withTheme(exports.POPOVER)(with_defaults_1.withDefaults({
    align: Popover_model_1.PopoverAlign.Left,
    placement: Popover_model_1.PopoverPlacement.Bottom,
})(RawPopover));
function stopPropagation(e) {
    e.stopPropagation();
}
function getArrowStyle(placement, align, offset) {
    switch (placement) {
        case Popover_model_1.PopoverPlacement.Top: //fallthrough
        case Popover_model_1.PopoverPlacement.Bottom: {
            switch (align) {
                case Popover_model_1.PopoverAlign.Left: {
                    return prefix_1.default({
                        left: offset,
                        transform: 'translateX(-50%)',
                    });
                }
                case Popover_model_1.PopoverAlign.Center: {
                    return prefix_1.default({
                        left: '50%',
                        transform: 'translateX(-50%)',
                    });
                }
                case Popover_model_1.PopoverAlign.Right: {
                    return prefix_1.default({
                        right: offset,
                        transform: 'translateX(50%)',
                    });
                }
            }
            break;
        }
        case Popover_model_1.PopoverPlacement.Left: //fallthrough
        case Popover_model_1.PopoverPlacement.Right: {
            switch (align) {
                case Popover_model_1.PopoverAlign.Top: {
                    return prefix_1.default({
                        top: offset,
                        transform: 'translateY(-50%)',
                    });
                }
                case Popover_model_1.PopoverAlign.Middle: {
                    return prefix_1.default({
                        top: '50%',
                        transform: 'translateY(-50%)',
                    });
                }
                case Popover_model_1.PopoverAlign.Bottom: {
                    return prefix_1.default({
                        bottom: offset,
                        transform: 'translateY(50%)',
                    });
                }
            }
        }
    }
    return undefined;
}
//# sourceMappingURL=Popover.js.map