"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var Scrollbar_1 = require("./Scrollbar");
var Scrollable_const_1 = require("../Scrollable/Scrollable.const");
var pure_1 = require("../../utils/pure");
var withTheme_1 = require("../../utils/withTheme");
exports.HORIZONTAL_SCROLLBAR = Symbol('HorizontalScrollbar');
var RawHorizontalScrollbar = /** @class */ (function (_super) {
    tslib_1.__extends(RawHorizontalScrollbar, _super);
    function RawHorizontalScrollbar() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._scrollTo = function (position) {
            if (_this._container) {
                _this._container.scrollLeft = position;
            }
        };
        ////////////////////////
        // DOM EVENT HANDLERS //
        ////////////////////////
        /**
         * @param {WheelEvent} event
         * @protected
         */
        _this.onTrackMouseWheel = function (event) {
            if (_this._container) {
                _this._container.scrollLeft += event.deltaX * 10 || event['detail'] * 10 || event['wheelDelta'] * -1;
            }
        };
        /**
         * @param {MouseEvent} event
         * @protected
         */
        _this.onTrackClick = function (event) {
            if (_this._track && _this._container) {
                var trackPosition = _this._track.getBoundingClientRect().left + window.pageXOffset - document.documentElement.clientLeft;
                _this._container.scrollLeft = Math.floor((event.clientX - trackPosition) / _this._ratio.position);
            }
        };
        /**
         * @param {MouseEvent} event
         * @protected
         */
        _this.onBarDragStart = function (event) {
            _this._previousDragCoordinate = event.clientX;
        };
        /**
         * @param {MouseEvent} event
         * @protected
         */
        _this.onBarDrag = function (event) {
            if (_this._container) {
                var delta = _this._previousDragCoordinate - event.clientX;
                _this._previousDragCoordinate = event.clientX;
                _this._container.scrollLeft -= delta / _this._ratio.position;
            }
        };
        return _this;
    }
    /**
     * @returns {Number}
     * @protected
     */
    RawHorizontalScrollbar.prototype._getMinBarSize = function () {
        return parseFloat(window.getComputedStyle(this._bar).getPropertyValue('min-width'));
    };
    /**
     * Hide native scroll
     * @private
     */
    RawHorizontalScrollbar.prototype._hideNativeScrollContainer = function () {
        var container = this.props.container;
        var height = this.context.size.height;
        if (height) {
            container.style.marginBottom = "-" + height + "px";
            container.style.height = "calc(100% + " + height + "px)";
        }
    };
    RawHorizontalScrollbar.prototype.componentDidMount = function () {
        _super.prototype.componentDidMount.call(this);
        this._minBarSize = this._getMinBarSize();
        this._hideNativeScrollContainer();
        this._update();
        if (this.props.scrollLeft) {
            this._scrollTo(this.props.scrollLeft);
        }
    };
    RawHorizontalScrollbar.prototype.componentWillReceiveProps = function (newProps) {
        if (this.props.scrollLeft !== newProps.scrollLeft) {
            if (newProps.scrollLeft) {
                this._scrollTo(newProps.scrollLeft);
            }
        }
    };
    RawHorizontalScrollbar.prototype._update = function () {
        this._toggle();
        this._ratio = this._getRatio();
        this._updateBar();
    };
    RawHorizontalScrollbar.prototype._toggle = function () {
        var container = this.props.container;
        var bounds = container.getBoundingClientRect();
        var visibleWidth = Math.round(bounds.width);
        var scrollWidth = container.scrollWidth;
        var isVisible = scrollWidth > visibleWidth;
        if (isVisible !== this.state.isVisible) {
            this.setState({
                isVisible: isVisible,
            });
            var emitter = this.context[Scrollable_const_1.SCROLLABLE_CONTEXT_EMITTER];
            emitter.emit(Scrollable_const_1.EVENT_SCROLABLE.SCROLLBAR_UPDATE, Scrollbar_1.SCROLLBAR_TYPE.HORIZONTAL, isVisible);
        }
    };
    RawHorizontalScrollbar.prototype._updateBar = function () {
        var container = this.props.container;
        var visibleWidth = container.clientWidth;
        var barWidth = Math.ceil(visibleWidth * this._ratio.size);
        this._bar.style.width = barWidth + "px";
        //position
        var scrollLeft = container.scrollLeft;
        var barPositionLeft = Math.floor(scrollLeft * this._ratio.position);
        this._bar.style.left = barPositionLeft + "px";
    };
    RawHorizontalScrollbar.prototype._getRatio = function () {
        if (this._container && this._track) {
            var _a = this._container, scrollSize = _a.scrollWidth, containerSize = _a.clientWidth;
            var trackSize = this._track.offsetWidth;
            var sizeRatio = trackSize / scrollSize;
            var possibleBarSize = containerSize * sizeRatio;
            var barSize = possibleBarSize < this._minBarSize ? this._minBarSize : possibleBarSize;
            var hiddenContentAreaSize = scrollSize - containerSize;
            var hiddenTrackAreaSize = trackSize - barSize;
            var positionRatio = hiddenTrackAreaSize / hiddenContentAreaSize;
            return {
                size: sizeRatio,
                position: positionRatio,
            };
        }
        return {
            size: 0,
            position: 0,
        };
    };
    RawHorizontalScrollbar = tslib_1.__decorate([
        pure_1.PURE
    ], RawHorizontalScrollbar);
    return RawHorizontalScrollbar;
}(Scrollbar_1.Scrollbar));
exports.RawHorizontalScrollbar = RawHorizontalScrollbar;
exports.HorizontalScrollbar = withTheme_1.withTheme(exports.HORIZONTAL_SCROLLBAR)(RawHorizontalScrollbar);
//# sourceMappingURL=HorizontalScrollbar.js.map