"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Option_1 = require("fp-ts/lib/Option");
var Setoid_1 = require("fp-ts/lib/Setoid");
var PeriodType;
(function (PeriodType) {
    PeriodType["AM"] = "AM";
    PeriodType["PM"] = "PM";
})(PeriodType = exports.PeriodType || (exports.PeriodType = {}));
var Section;
(function (Section) {
    Section[Section["Hours"] = 0] = "Hours";
    Section[Section["Minutes"] = 1] = "Minutes";
    Section[Section["Seconds"] = 2] = "Seconds";
    Section[Section["PeriodType"] = 3] = "PeriodType";
})(Section = exports.Section || (exports.Section = {}));
exports.MAX_VALID_MINS_AND_SEC = 59;
exports.MAX_VALID_HOURS_FOR_24H_FORMAT = 23;
exports.MAX_VALID_HOURS_FOR_12H_FORMAT = 12;
exports.EMPTY_SECTION = '--';
var setoidStrict = { equals: Setoid_1.strictEqual };
var setoidPeriodType = setoidStrict;
var setoidOptionNumber = Option_1.getSetoid(Setoid_1.setoidNumber);
var setoidOptionPeriodType = Option_1.getSetoid(setoidPeriodType);
var timeSetoid = Setoid_1.getRecordSetoid({
    hours: setoidOptionNumber,
    minutes: setoidOptionNumber,
    seconds: setoidOptionNumber,
    periodType: setoidOptionPeriodType,
});
exports.isTimesDifferent = function (x, y) {
    return !timeSetoid.equals(x, y);
};
exports.formatNumericValue = function (value) {
    if (value < 10) {
        return "0" + value;
    }
    return "" + value;
};
exports.formatTimePeriod = function (periodType) {
    switch (periodType) {
        case PeriodType.AM:
            return 'am';
        case PeriodType.PM:
            return 'pm';
    }
};
/**
 * Values can be zeros (start from 0). Max is included value.
 */
function add(a, b, max) {
    return a
        .map(function (a) {
        var rawResult = (a + b) % (max + 1);
        return rawResult < 0 ? rawResult + max + 1 : rawResult;
    })
        .alt(Option_1.some(b < 0 ? max : 0));
}
exports.add = add;
function isDefined(value) {
    return typeof value !== 'undefined';
}
exports.isDefined = isDefined;
function findActiveSectionOnKeyLeft(activeState, isSecondsExist) {
    switch (activeState) {
        case Section.PeriodType: {
            return isSecondsExist ? Section.Seconds : Section.Minutes;
        }
        case Section.Seconds: {
            return Section.Minutes;
        }
        case Section.Minutes: {
            return Section.Hours;
        }
        default:
            return Section.Hours;
    }
}
exports.findActiveSectionOnKeyLeft = findActiveSectionOnKeyLeft;
function findActiveSectionOnKeyRight(activeSection, isSecondsExist, isClockFormatExist) {
    switch (activeSection) {
        case Section.Hours: {
            return Section.Minutes;
        }
        case Section.Minutes: {
            if (isSecondsExist) {
                return Section.Seconds;
            }
            else if (isClockFormatExist) {
                return Section.PeriodType;
            }
            else {
                return Section.Minutes;
            }
        }
        case Section.Seconds: {
            return isClockFormatExist ? Section.PeriodType : Section.Seconds;
        }
        case Section.PeriodType: {
            return Section.PeriodType;
        }
        default:
            return Section.Hours;
    }
}
exports.findActiveSectionOnKeyRight = findActiveSectionOnKeyRight;
function togglePeriodType(periodType) {
    var periodTypeNormalized = periodType.getOrElse(PeriodType.AM);
    switch (periodTypeNormalized) {
        case PeriodType.AM:
            return Option_1.some(PeriodType.PM);
        case PeriodType.PM:
            return Option_1.some(PeriodType.AM);
    }
}
exports.togglePeriodType = togglePeriodType;
exports.renderSection = function (time) {
    return time.getOrElse(exports.EMPTY_SECTION);
};
//# sourceMappingURL=TimeInput.model.js.map