"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var _a;
var React = require("react");
// import {
//     Children, createElement, ComponentClass, Component, Ref,
//     ComponentType
// } from 'react';
var PropTypes = require("prop-types");
var THEME_CONTEXT_KEY = '@@dx-util/withTheme-context-key'; //should be serializable
var THEME_CONFIG_KEY = '@@dx-util/withTheme-config-key';
exports.withTheme = function (name, defaultTheme) {
    if (defaultTheme === void 0) { defaultTheme = {}; }
    function decorate(Target) {
        var _a;
        if (Target['config'] && Target['config'].name === name) {
            //already wrapped - just merge in new defaultTheme
            Target['config'].theme = mergeTwo(Target['config'].theme, defaultTheme);
            return Target;
        }
        // noinspection UnnecessaryLocalVariableJS
        var config = {
            name: name,
            theme: defaultTheme,
        };
        var Themed = /** @class */ (function (_super) {
            tslib_1.__extends(Themed, _super);
            function Themed() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            Themed.prototype.render = function () {
                var themr = this.context.themr && this.context.themr.theme && this.context.themr.theme[name];
                /* BROKEN TYPES HERE
                * the next 4 lines may be replaced by `const { withRef, theme, ...rest }= this.props;`
                * but TS cannot get `rest` type from generic `this.props` type
                * details:
                * issue https://github.com/Microsoft/TypeScript/issues/10727
                * PR https://github.com/Microsoft/TypeScript/pull/13288
                */
                var _a = this.props, withRef = _a.withRef, theme = _a.theme;
                var rest = Object.assign({}, this.props);
                delete rest.withRef;
                delete rest.theme;
                var props = tslib_1.__assign({}, rest, { ref: withRef, theme: mergeThemes(config.theme, themr, this.context[THEME_CONTEXT_KEY.toString()][name], (theme ||
                        {})) });
                return React.createElement(Target, props);
            };
            Themed.displayName = "Themed(" + (Target.displayName || Target.name) + ")";
            Themed.contextTypes = (_a = {},
                _a[THEME_CONTEXT_KEY.toString()] = PropTypes.object.isRequired,
                //legacy react-css-themr context for backward compatibility
                _a.themr = PropTypes.shape({
                    theme: PropTypes.object.isRequired,
                }),
                _a);
            return Themed;
        }(React.Component));
        Themed[THEME_CONFIG_KEY] = config;
        return Themed; // type does not match because of `rest` issue above
    }
    return decorate;
};
/**
 * Merges passed themes by concatenating string keys and processing nested themes
 */
function mergeThemes() {
    var themes = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        themes[_i] = arguments[_i];
    }
    return themes.reduce(function (acc, theme) { return mergeTwo(acc, theme); }, {});
}
exports.mergeThemes = mergeThemes;
function mergeTwo(original, mixin) {
    if (original === void 0) { original = {}; }
    if (mixin === void 0) { mixin = {}; }
    //make a copy to avoid mutations of nested objects
    //also strip all functions injected by isomorphic-style-loader
    var result = Object.keys(original).reduce(function (acc, key) {
        var value = original[key];
        if (typeof value !== 'function') {
            acc[key] = value;
        }
        return acc;
    }, {});
    //traverse mixin keys and merge them to resulting theme
    Object.keys(mixin).forEach(function (key) {
        //there's no need to set any defaults here
        var originalValue = result[key];
        var mixinValue = mixin[key];
        switch (typeof mixinValue) {
            case 'object': {
                //possibly nested theme object
                switch (typeof originalValue) {
                    case 'object': {
                        //exactly nested theme object - go recursive
                        result[key] = mergeThemes(originalValue, mixinValue);
                        break;
                    }
                    case 'undefined': {
                        //original does not contain this nested key - just take it as is
                        result[key] = mixinValue;
                        break;
                    }
                    default: {
                        //can't merge an object with a non-object
                        throw new Error("You are merging object " + key + " with a non-object " + originalValue);
                    }
                }
                break;
            }
            case 'undefined': //fallthrough - handles accidentally unset values which may come from props
            case 'function': {
                //this handles issue when isomorphic-style-loader addes helper functions to css-module
                break; //just skip
            }
            default: {
                //plain values
                switch (typeof originalValue) {
                    case 'object': {
                        //can't merge a non-object with an object
                        throw new Error("You are merging non-object " + mixinValue + " with an object " + key);
                    }
                    case 'undefined': {
                        //mixin key is new to original theme - take it as is
                        result[key] = mixinValue;
                        break;
                    }
                    case 'function': {
                        //this handles issue when isomorphic-style-loader addes helper functions to css-module
                        break; //just skip
                    }
                    default: {
                        //finally we can merge
                        result[key] = originalValue
                            .split(' ')
                            .concat(mixinValue.split(' '))
                            .filter(function (item, pos, self) { return self.indexOf(item) === pos && item !== ''; })
                            .join(' ');
                        break;
                    }
                }
                break;
            }
        }
    });
    return result;
}
var ThemeProvider = /** @class */ (function (_super) {
    tslib_1.__extends(ThemeProvider, _super);
    function ThemeProvider() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ThemeProvider.prototype.render = function () {
        return React.Children.only(this.props.children);
    };
    ThemeProvider.prototype.getChildContext = function () {
        var _a;
        return _a = {},
            _a[THEME_CONTEXT_KEY.toString()] = this.props.theme,
            _a;
    };
    ThemeProvider.childContextTypes = (_a = {},
        _a[THEME_CONTEXT_KEY.toString()] = PropTypes.object,
        _a);
    return ThemeProvider;
}(React.Component));
exports.ThemeProvider = ThemeProvider;
//# sourceMappingURL=withTheme.js.map